@@ -1,11 +1,12 @@
 from .crawler import Crawler
-from .http import CachedWeb, Web
+from .http import CachedWeb, FetchError, Web
 from .link import EggLink, Link, SourceLink
 
 __all__ = (
   CachedWeb,
   Crawler,
   EggLink,
+  FetchError,
   Link,
   SourceLink,
   Web,
