@@ -6,7 +6,7 @@ import sys
 import tempfile
 
 from .common import safe_mkdtemp, safe_rmtree
-from .interpreter import PythonInterpreter
+from .interpreter import PythonInterpreter, PythonCapability
 from .tracer import TRACER
 
 from pkg_resources import Distribution, PathMetadata
@@ -27,16 +27,16 @@ def after_installation(function):
 
 
 class InstallerBase(object):
-  SETUP_BOOTSTRAP = """
-if '%(setuptools_path)s':
-  import sys
-  sys.path.insert(0, '%(setuptools_path)s')
-  import setuptools
-__file__ = '%(setup_py)s'
+  SETUP_BOOTSTRAP_HEADER = "import sys"
+  SETUP_BOOTSTRAP_MODULE = "sys.path.insert(0, %(path)r); import %(module)s"
+  SETUP_BOOTSTRAP_FOOTER = """
+__file__ = 'setup.py'
 exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
 """
 
-  class InstallFailure(Exception): pass
+  class Error(Exception): pass
+  class InstallFailure(Error): pass
+  class IncapableInterpreter(Error): pass
 
   def __init__(self, source_dir, strict=True, interpreter=None, install_dir=None):
     """
@@ -50,6 +50,20 @@ exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
     self._installed = None
     self._strict = strict
     self._interpreter = interpreter or PythonInterpreter.get()
+    if not self._interpreter.satisfies(self.capability) and strict:
+      raise self.IncapableInterpreter('Interpreter %s not capable of running %s' % (
+          self._interpreter, self.__class__.__name__))
+
+  def mixins(self):
+    """Return a map from import name to requirement to load into setup script prior to invocation.
+
+       May be subclassed.
+    """
+    return {}
+
+  @property
+  def install_tmp(self):
+    return self._install_tmp
 
   def _setup_command(self):
     """the setup command-line to run, to be implemented by subclasses."""
@@ -58,20 +72,27 @@ exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
   def _postprocess(self):
     """a post-processing function to run following setup.py invocation."""
 
+  @property
+  def capability(self):
+    """returns the PythonCapability necessary for the interpreter to run this installer."""
+    return PythonCapability(self.mixins().values())
+
+  @property
+  def bootstrap_script(self):
+    bootstrap_modules = []
+    for module, requirement in self.mixins().items():
+      path = self._interpreter.get_location(requirement)
+      if not path:
+        assert not self._strict  # This should be caught by validation
+        continue
+      bootstrap_modules.append(self.SETUP_BOOTSTRAP_MODULE % {'path': path, 'module': module})
+    return '\n'.join(
+        [self.SETUP_BOOTSTRAP_HEADER] + bootstrap_modules + [self.SETUP_BOOTSTRAP_FOOTER])
+
   def run(self):
     if self._installed is not None:
       return self._installed
 
-    if self._interpreter.distribute is None and self._strict:
-      self._installed = False
-      print('Failed to find distribute in sys.path!', file=sys.stderr)
-      return self._installed
-
-    setup_bootstrap = Installer.SETUP_BOOTSTRAP % {
-      'setuptools_path': self._interpreter.distribute or '',
-      'setup_py': 'setup.py'
-    }
-
     with TRACER.timed('Installing %s' % self._install_tmp, V=2):
       command = [self._interpreter.binary, '-']
       command.extend(self._setup_command())
@@ -79,8 +100,9 @@ exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
           stdin=subprocess.PIPE,
           stdout=subprocess.PIPE,
           stderr=subprocess.PIPE,
+          env=self._interpreter.sanitized_environment(),
           cwd=self._source_dir)
-      so, se = po.communicate(setup_bootstrap.encode('ascii'))
+      so, se = po.communicate(self.bootstrap_script.encode('ascii'))
       self._installed = po.returncode == 0
 
     if not self._installed:
@@ -112,17 +134,16 @@ class Installer(InstallerBase):
       >>> tornado_distribution.activate()
       >>> import tornado
 
-    Alternately you can pass the distribution to a Distiller object and convert it to an egg:
-      >>> from twitter.common.python.distiller import Distiller
-      >>> Distiller(tornado_distribution).distill()
+    Alternately you can use the EggInstaller to create an egg instead:
+      >>> from twitter.common.python.installer import EggInstaller
+      >>> EggInstaller(tornado_tgz.fetch()).bdist()
       '/var/folders/Uh/UhXpeRIeFfGF7HoogOKC+++++TI/-Tmp-/tmpufgZOO/tornado-2.3-py2.6.egg'
   """
-
   def __init__(self, source_dir, strict=True, interpreter=None):
     """
       Create an installer from an unpacked source distribution in source_dir.
 
-      If strict=True, fail if any installation dependencies (e.g. distribute)
+      If strict=True, fail if any installation dependencies (e.g. setuptools)
       are missing.
     """
     super(Installer, self).__init__(source_dir, strict=strict, interpreter=interpreter)
@@ -179,17 +200,67 @@ class Installer(InstallerBase):
     return Distribution.from_location(base_dir, os.path.basename(egg_info), metadata=metadata)
 
 
-class Packager(InstallerBase):
+class DistributionPackager(InstallerBase):
+  def find_distribution(self):
+    dists = os.listdir(self.install_tmp)
+    if len(dists) == 0:
+      raise self.InstallFailure('No distributions were produced!')
+    elif len(dists) > 1:
+      raise self.InstallFailure('Ambiguous source distributions found: %s' % (' '.join(dists)))
+    else:
+      return os.path.join(self.install_tmp, dists[0])
+
+
+class Packager(DistributionPackager):
   """
     Create a source distribution from an unpacked setup.py-based project.
   """
-
   def _setup_command(self):
     return ['sdist', '--formats=gztar', '--dist-dir=%s' % self._install_tmp]
 
   @after_installation
   def sdist(self):
-    dists = os.listdir(self._install_tmp)
-    if len(dists) != 1:
-      raise self.InstallFailure('Ambiguous source distributions found.')
-    return os.path.join(self._install_tmp, dists[0])
+    return self.find_distribution()
+
+
+class EggInstaller(DistributionPackager):
+  """
+    Create a source distribution from an unpacked setup.py-based project.
+  """
+  MIXINS = {
+      'setuptools': 'setuptools>=1',
+  }
+
+  def mixins(self):
+    mixins = super(EggInstaller, self).mixins().copy()
+    mixins.update(self.MIXINS)
+    return mixins
+
+  def _setup_command(self):
+    return ['bdist_egg', '--dist-dir=%s' % self._install_tmp]
+
+  @after_installation
+  def bdist(self):
+    return self.find_distribution()
+
+
+class WheelInstaller(DistributionPackager):
+  """
+    Create a source distribution from an unpacked setup.py-based project.
+  """
+  MIXINS = {
+      'setuptools': 'setuptools>=2',
+      'wheel': 'wheel>=0.17',
+  }
+
+  def mixins(self):
+    mixins = super(WheelInstaller, self).mixins().copy()
+    mixins.update(self.MIXINS)
+    return mixins
+
+  def _setup_command(self):
+    return ['bdist_wheel', '--dist-dir=%s' % self._install_tmp]
+
+  @after_installation
+  def bdist(self):
+    return self.find_distribution()
