@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==================================================================================================
+
 from __future__ import absolute_import
 
 import logging
@@ -84,7 +85,7 @@ class PEXBuilder(object):
   def clone(self, into=None):
     chroot_clone = self._chroot.clone(into=into)
     return PEXBuilder(chroot=chroot_clone, interpreter=self._interpreter,
-                      pex_info=PexInfo(content=self._pex_info.dump()))
+                      pex_info=self._pex_info.copy())
 
   def path(self):
     return self.chroot().path()
@@ -196,8 +197,8 @@ class PEXBuilder(object):
       Write enough of distribute into the .pex .bootstrap directory so that
       we can be fully self-contained.
     """
-    distribute = dist_from_egg(self._interpreter.distribute)
-    for fn, content_stream in DistributionHelper.walk_data(distribute):
+    setuptools = dist_from_egg(self._interpreter.get_location('setuptools'))
+    for fn, content_stream in DistributionHelper.walk_data(setuptools):
       if fn == 'pkg_resources.py':
         self._chroot.write(content_stream.read(),
             os.path.join(self.BOOTSTRAP_DIR, 'pkg_resources.py'), 'resource')
